
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#
#

include logging
logdebug "Load mutex package"

if onluke; then
  include lecaluke
else
  LOCK_DIR="${HOME}/lecalock"
fi

# Create the lockdir if it does not exist
if [[ ! -d "${LOCK_DIR}" ]]; then
	mkdir -p "${LOCK_DIR}"
	chmod a+rwxt "${LOCK_DIR}"
fi

include clobber

LECA_LOCK_DELAY=2
LECA_LOCK_MAXRETRY=0

function lockfile() {
  echo "${LOCK_DIR}/__${1}__.lecalock"
}

function locktag() {
  echo "$(hostname).$$"
}

function getlock() {
  local name="$1"
  shift

  if [[ ! -z "${1}" ]]; then
    mode="${1}"
    if [[ "$mode" != "global" ]]; then
      mode="user"
    fi
  else
    mode="user"
  fi

  if [[ "${mode}" == "global" ]]; then
    name=$(lockfile "${name}")
  else
    name=$(lockfile "${name}.$(whoami)")
  fi

  local TAG=$(locktag)
  local retry=0

  notgetlock=1

  while (( notgetlock > 0 && ( LECA_LOCK_MAXRETRY == 0 || retry > LECA_LOCK_MAXRETRY) )); do

    pushnoclobber
      echo "${TAG}" 2>/dev/null 1> "${name}"
      notgetlock=$?
    popnoclobber

    if (( notgetlock > 0 )); then
      if [[ $(cat "${name}" 2> /dev/null) == "${TAG}" ]]; then
        notgetlock=0
      fi
    fi

    if (( notgetlock > 0 )); then
      sleep "${LECA_LOCK_DELAY}"
      retry=$((retry + 1))
    fi

  done

  if (( notgetlock > 0 )); then
    return $notgetlock
  else
  	chmod a+w "${name}"
    return 0
  fi
}

function releaselock() {
  local name="$1"
  shift

  if [[ ! -z "${1}" ]]; then
    mode="${1}"
    if [[ "$mode" != "global" ]]; then
      mode="user"
    fi
  else
    mode="user"
  fi

  if [[ "${mode}" == "global" ]]; then
    name=$(lockfile "${name}")
  else
    name=$(lockfile "${name}.$(whoami)")
  fi

  local TAG=$(locktag)
  local LOCK=""

  if [[ -f "${name}" ]]; then
    LOCK=$(cat "${name}" 2> /dev/null)
  else
    return 0
  fi


  if [[ "${LOCK}" == "${TAG}" ]]; then
    rm -f "${name}"
    return 0
  fi

  return 1
}

function releaseoldlock() {
	local minutes=10
	
	if [[ ! -z $1 ]]; then
		minutes="$1"
	fi
	
	for lock in $(find "${LOCK_DIR}" -type f -cmin "+$1" -print); do
		rm "${lock}"
		loginfo "releasing old lock: ${lock}"
	done

}

